package uk.ac.ox.eolo.model;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.semanticweb.HermiT.model.Atom;
import org.semanticweb.HermiT.model.AtomicConcept;
import org.semanticweb.HermiT.model.Individual;
import org.semanticweb.HermiT.model.Term;
import org.semanticweb.HermiT.model.Variable;

import uk.ac.ox.eolo.URIUtils;
import uk.ac.ox.eolo.exception.IllegalInputQueryException;
import uk.ac.ox.eolo.model.ConjunctiveQuery;
import uk.ac.ox.eolo.cqparser.ConjunctiveQueryParser;
import uk.ac.ox.cs.JRDFox.Prefixes;


public class ConjunctiveQuery {

	
	protected  Atom[] m_queryAtoms;
    protected  Term[] m_answerTerms;
    protected  Term[] m_resultBuffer;
    protected Prefixes prefixes;

    public ConjunctiveQuery(Atom[] queryAtoms,Term[] answerTerms) {
        m_queryAtoms=queryAtoms;
        m_answerTerms=answerTerms;
        m_resultBuffer=answerTerms.clone();
        prefixes = new Prefixes();
    }
    
    
    public void clear() {
    	m_queryAtoms = null;
    	m_answerTerms = null;
    	m_resultBuffer = null;
    	prefixes = null;
    }
    
    public Prefixes getPrefixes() {
    	return this.prefixes;
    }
    
    public Term[] getAnswerTerms() {
		return m_answerTerms;
	}
    
    public ConjunctiveQuery(Atom[] queryAtoms,Term[] answerTerms, Prefixes prefixes) {
        m_queryAtoms=queryAtoms;
        m_answerTerms=answerTerms;
        m_resultBuffer=answerTerms.clone();
        this.prefixes = prefixes;
    }
    
    public boolean isAnswerVariable(Term t ) {
    	if (t instanceof Variable) {
			for (Term x : this.m_answerTerms)
				if (x.equals(t))
					return true;
				return true;
		}
    	return false;
    }
    
    public boolean hasQuantifiedVariables() {
    	for (Term t : this.getTerms()) 
    		if ((t instanceof Variable) && !isAnswerVariable(t))
    			return true;
    	return false;
    }
    
    public List<Term> quantifiedVariables() {
    	List<Term> vars = new ArrayList<Term>();
    	for (Term t : getTerms()) 
    		if ((t instanceof Variable) && !isAnswerVariable(t))    	
    			vars.add(t);
    	return vars;
    }

    public int getNumberOfQueryAtoms() {
        return m_queryAtoms.length;
    }
    
    public int getNumberOfUnaryAtoms() {
    	return getNumberOfQueryAtoms() - getNumberOfBinaryAtoms();
    }
    
    public int getNumberOfBinaryAtoms() {
        int i =0;
        for (Atom a: m_queryAtoms) {
        	if (a.getArity() >1)
        		i++;
        }
        return i;
    }
    
    
    public boolean containsBinaryAtoms() {
        for (Atom a: m_queryAtoms) {
        	if (a.getArity() >1)
        		return true;
        }
        return false;
    }
    
    public Atom getQueryAtom(int atomIndex) {
        return m_queryAtoms[atomIndex];
    }
    public int getNumberOfAnswerTerms() {
        return m_answerTerms.length;
    }
    public Term getAnswerTerm(int termIndex) {
        return m_answerTerms[termIndex];
    }
   

    public String toString() {
    	String res = EoloPrefixes.getDeclaration();
    	Map<String, String> iriMap = prefixes.getPrefixIRIsByPrefixName();
    	for (String shortIri: iriMap.keySet()) {
    		res += "PREFIX " + shortIri + " <"+ iriMap.get(shortIri)  + ">\n";
    	}
    	res += "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n";
    	res += "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n";
    	res += "select ";
    	if (m_answerTerms.length ==0 )
    		res += " <a> ";
    	else {
    		for (Term t: m_answerTerms)
    			res+= " " + t + " ";
    	}
    	res += " WHERE { ";
    	for (Atom a : m_queryAtoms){
    		if (a.getArity() == 1)
    			res+= a.getArgument(0).toString() + " rdf:type " + URIUtils.removeBrackets(a.getDLPredicate().toString()) + " . ";
    		if (a.getArity() == 2)
    			res+= a.getArgument(0).toString() + "  " + URIUtils.removeBrackets(a.getDLPredicate().toString()) + " " + a.getArgument(1) + " . ";
    	}
    	return res + "}";
    	
    }
    
    public String toString(boolean prefix) {
    	String res = "";
    	if (prefix)  {
    		EoloPrefixes.getDeclaration();
    		Map<String, String> iriMap = prefixes.getPrefixIRIsByPrefixName();
        	for (String shortIri: iriMap.keySet()) {
        		res += "PREFIX " + shortIri + " <"+ iriMap.get(shortIri)  + ">\n";
        	}
        	res += "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n";
        	res += "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n";
    	}
    	res += "select ";
    	if (m_answerTerms.length ==0 )
    		res += " <a> ";
    	else {
    		for (Term t: m_answerTerms)
    			res+= " " + t + " ";
    	}
    	res += " WHERE { ";
    	for (Atom a : m_queryAtoms){
    		if (a.getArity() == 1)
    			res+= a.getArgument(0).toString() + " rdf:type " + URIUtils.removeBrackets(a.getDLPredicate().toString()) + " . ";
    		if (a.getArity() == 2)
    			res+= a.getArgument(0).toString() + "  " + URIUtils.removeBrackets(a.getDLPredicate().toString()) + " " + a.getArgument(1) + " . ";
    	}
    	return res + "}";
    	
    }
    
    /**
     * 
     * @param sub
     * @return
     */
    public ConjunctiveQuery applySubstitution(Map<Term, Term> sub) {
			Term[] answerTerms = new Term[0];
			Atom[] atoms = new Atom[this.getAtoms().length];
			for (int i = 0; i <  this.getAtoms().length; i++) {
				Atom a = this.getAtoms()[i];
				if (a.getArity() ==1)  {
					Term s = (sub.containsKey(a.getArgument(0))) ? sub.get(a.getArgument(0)) : a.getArgument(0);
					atoms[i] = Atom.create(a.getDLPredicate(), s);
				} else {
					Term s = (sub.containsKey(a.getArgument(0))) ? sub.get(a.getArgument(0)) : a.getArgument(0);
					Term t =  (sub.containsKey(a.getArgument(1))) ? sub.get(a.getArgument(1)) : a.getArgument(1);
					atoms[i] = Atom.create(a.getDLPredicate(), s, t);
				}
			}
			return new ConjunctiveQuery(atoms, answerTerms, this.prefixes);
 
	}
    
    
    private Term createTerm(String ind) {
    	ind = ind.replaceAll("<", "");
    	return Individual.create(ind.replaceAll(">", ""));
    }
    /**
     * 
     * @param sub
     * @return
     */
    public ConjunctiveQuery applyAnswer(Map<Term, String> sub) {
		Term[] answerTerms = new Term[0];
		List<Atom> list = new ArrayList<Atom>();
		for (Atom a: this.getAtoms()) {
			if (a.getArity() ==1)  {
				Term s = (sub.containsKey(a.getArgument(0))) ? createTerm(sub.get(a.getArgument(0))) : a.getArgument(0);
				list.add(Atom.create(a.getDLPredicate(), s));
			} else {
				Term s = (sub.containsKey(a.getArgument(0))) ? createTerm(sub.get(a.getArgument(0))) : a.getArgument(0);
				Term t =  (sub.containsKey(a.getArgument(1))) ? createTerm(sub.get(a.getArgument(1))) : a.getArgument(1);
				list.add(Atom.create(a.getDLPredicate(), s, t));
			}
		}
		return new ConjunctiveQuery(list.toArray(new Atom[list.size()]), answerTerms, prefixes);
	}

    public static ConjunctiveQuery parse(String query) throws FileNotFoundException, IllegalInputQueryException, IOException {
    	return (new ConjunctiveQueryParser(query)).parse();
    }


	public Atom[] getAtoms() {
		return this.m_queryAtoms;
	}
	
	public   Term[] getTerms() {
		Set<Term> terms = new LinkedHashSet<Term>();
		for (Atom a : m_queryAtoms) {
			terms.add(a.getArgument(0));
			if (a.getArity()> 1)
				terms.add(a.getArgument(1));
		}
		return terms.toArray(new Term[terms.size()]);
	}


	
}
