package uk.ac.ox.eolo.model;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.semanticweb.HermiT.model.Atom;
import org.semanticweb.HermiT.model.AtomicConcept;
import org.semanticweb.HermiT.model.Individual;
import org.semanticweb.HermiT.model.Term;
import org.semanticweb.HermiT.model.Variable;
import uk.ac.ox.eolo.model.ConjunctiveQuery;
import uk.ac.ox.cs.JRDFox.Prefixes;



public class ExtendedConjunctiveQuery extends ConjunctiveQuery {

	private Term[] ansTerms;

	public ExtendedConjunctiveQuery(Atom[] queryAtoms, Term[] answerTerms, Prefixes pref) {
		super(queryAtoms, getExtendedHead(queryAtoms, answerTerms), pref);
		this.ansTerms = answerTerms.clone(); 
	}

	public int getNumberOfRealAnswerTerms() {
		return ansTerms.length;
	}
	
	public boolean isAnswerTerm(Term t) {
		for (Term ans : this.m_answerTerms)
			if (ans.equals(t))
				return true;
		return false;
	}
	
	public boolean isRealAnswerTerm(Term t) {
		for (Term ans : this.ansTerms)
			if (ans.equals(t))
				return true;
		return false;
	}

	public Term getRealAnswerTerm(int termIndex) {
		return ansTerms[termIndex];
	}

	public int getNumberOfTerms() {
		return this.getTerms().length;
	}


	
	public Atom[] getAtoms() {
		return m_queryAtoms;
	}
	
	public Term[] getRealAnswerTerms() {
		return ansTerms;
	}
	
	public Term[] getIndividualsInBody() {
		List<Term> list = new ArrayList<Term>();
		for (Term t : this.getTerms()) {
			if (!isAnswerTerm(t))
				list.add(t);
		}
		return list.toArray(new Term[list.size()]);
	}

	private static  Term[] getExtendedHead(Atom[] queryAtoms, Term[] answerTerms) {
		List<Term> terms = new LinkedList<Term>();
		for (Term t :answerTerms) {
			terms.add(t);
		}
		for (Atom a : queryAtoms) {
			if (a.getArgument(0) instanceof Variable && !terms.contains(a.getArgument(0)))
				terms.add(a.getArgument(0));
			if (a.getArity()> 1 && a.getArgument(1) instanceof Variable && !terms.contains(a.getArgument(1)))
				terms.add(a.getArgument(1));
		}
		return terms.toArray(new Term[terms.size()]);
		
	}
	
	
	
	
	
	
	/**
	 * Given a CQ q, it computes a CQ q' whose answer variable are the set of all variables
	 * in q; moreover, q' contains an atom Ind(x) for each answer variable of q.
	 * @param q
	 * @return
	 */
	public static ExtendedConjunctiveQuery computeExtension(ConjunctiveQuery q) {
		List<Term> answerTerms = new ArrayList<Term>();
		List<Atom> atoms = new ArrayList<Atom>();
		for (int i = 0; i < q.getNumberOfAnswerTerms(); i++) {
			answerTerms.add(q.getAnswerTerm(i));
			if (q.getAnswerTerm(i) instanceof Variable) 
				atoms.add(Atom.create(AtomicConcept.create(EoloPrefixes.IND_PREFIX
						+ "Ind"), q.getAnswerTerm(i)));
		}
		for (int i = 0; i < q.getNumberOfQueryAtoms(); i++) {
			atoms.add(q.getQueryAtom(i));
		}
		return new ExtendedConjunctiveQuery(atoms.toArray(new Atom[atoms.size()]), answerTerms.toArray(new Term[answerTerms.size()]), q.prefixes);
		
	}

	public Term getTerm(int i) {
		return this.getTerm(i);
	}
	
	public ConjunctiveQuery ground(Map<Term, String> tau) {
		Term[] answerTerms = {};
		List<Atom> list = new ArrayList<Atom>();
		for (Atom a: this.getAtoms()) {
			if (a.getArity()==1) {
				String tau_t = tau.get(a.getArgument(0)).replaceAll("<", "");
				tau_t = tau_t.replaceAll(">", "");
				Term t = (isRealAnswerTerm(a.getArgument(0))) ? Individual.create(tau_t) : a.getArgument(0);
				list.add(Atom.create(a.getDLPredicate(), t));
			} else {
				String tau_s = tau.get(a.getArgument(0)).replaceAll("<", "");
				tau_s =tau_s.replaceAll(">", "");
				String tau_t = tau.get(a.getArgument(1)).replaceAll("<", "");
				tau_t =tau_t.replaceAll(">", "");
				Term s = (isRealAnswerTerm(a.getArgument(0))) ? Individual.create(tau_s) : a.getArgument(0);
				Term t = (isRealAnswerTerm(a.getArgument(1))) ? Individual.create(tau_t) : a.getArgument(1);
				list.add(Atom.create(a.getDLPredicate(), s,t));
			}
			
		}
		for (Term t : this.getTerms()) {
			if (t instanceof Variable && !isRealAnswerTerm(t))
				list.add(Atom.create(AtomicConcept.create("eolo-ind:Ind"), t));
		}
		return new ConjunctiveQuery(list.toArray(new Atom[list.size()]), answerTerms, prefixes);
	}

}
