package uk.ac.ox.eolo.model;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.semanticweb.HermiT.model.AtomicRole;
import org.semanticweb.HermiT.model.Role;
import org.semanticweb.owlapi.model.AxiomType;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLSubObjectPropertyOfAxiom;
import org.semanticweb.owlapi.model.OWLTransitiveObjectPropertyAxiom;

import uk.ac.ox.cs.JRDFox.Prefixes;
import uk.ac.ox.eolo.URIUtils;

public class RoleHierarchy {
	
	private Map<String, Set<String>> roleHierarchy;
	private Set<String> transitiveRoles;
	
	public RoleHierarchy() {
		roleHierarchy= new HashMap<String, Set<String>>();
		transitiveRoles = new LinkedHashSet<String>();
	}
	
	
	public Set<String> getRoles() {
		return roleHierarchy.keySet();
	}
	
	
	public void addRoleISA(String subRole, String superRole) {
		if (roleHierarchy.containsKey(superRole))
			roleHierarchy.get(superRole).add(subRole);
		else {
			Set<String> s = new LinkedHashSet<String>();
			s.add(subRole);
			roleHierarchy.put(superRole, s);
		}
	}
	
	
	public boolean addTransitiveRole(String transRole) {
		return transitiveRoles.add(transRole);
	}
	
	
	public Set<String> getTransitive() {
		return this.transitiveRoles;
	}
	
	
	public Set<String> subRoles(String r) {
		Set<String> subroles = new LinkedHashSet<String>();
		if (!roleHierarchy.containsKey(r)) {
			return null;
		}
		for (String dirSubRole : roleHierarchy.get(r)) {
			subroles.add(dirSubRole);
			if (!dirSubRole.toString().equals(r.toString()))
				subroles.addAll(subRoles(dirSubRole));
		}
		return subroles;
	}
	
	public boolean isSimple(String r) throws Exception {
		for (String s : this.subRoles(r)) {
			if (transitiveRoles.contains(s))
				return false;
		}
		return true;
	}
	
	public boolean isTransitive(String r) {
		return transitiveRoles.contains(r);
	}
	
	public boolean isaHolds(String subrole, String superRole) {
		return subRoles(superRole).contains(subrole);
	}
	
	public  boolean allTransitive(Set<String> L) {
		for (String r : L)
			if (!this.isTransitive(r))
				return false;
		return true;
	}


	public static RoleHierarchy populate(OWLOntology shoelOntology) {
		RoleHierarchy roleISA = new RoleHierarchy();
		for (OWLObjectProperty p : shoelOntology.getObjectPropertiesInSignature()) {
			roleISA.addRoleISA(p.asOWLObjectProperty().getIRI().toString(), p.asOWLObjectProperty().getIRI().toString());
		}
		
		for (OWLSubObjectPropertyOfAxiom ax : shoelOntology.getAxioms(AxiomType.SUB_OBJECT_PROPERTY)) {
			String superRole = ax.getSuperProperty().asOWLObjectProperty().getIRI().toString();
			String subrole =  ax.getSubProperty().asOWLObjectProperty().getIRI().toString();
			roleISA.addRoleISA(subrole, superRole);
		}
		for (OWLTransitiveObjectPropertyAxiom ax : shoelOntology.getAxioms(AxiomType.TRANSITIVE_OBJECT_PROPERTY))
			roleISA.addTransitiveRole(ax.getProperty().asOWLObjectProperty().getIRI().toString());
		return roleISA;
	}
	
	
	


}
