package uk.ac.ox.eolo.profile;


import java.util.HashSet;
import java.util.Set;

import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLDataHasValue;
import org.semanticweb.owlapi.model.OWLDataIntersectionOf;
import org.semanticweb.owlapi.model.OWLDataOneOf;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLDataPropertyAssertionAxiom;
import org.semanticweb.owlapi.model.OWLDataPropertyDomainAxiom;
import org.semanticweb.owlapi.model.OWLDataSomeValuesFrom;
import org.semanticweb.owlapi.model.OWLEquivalentDataPropertiesAxiom;
import org.semanticweb.owlapi.model.OWLFunctionalDataPropertyAxiom;
import org.semanticweb.owlapi.model.OWLHasKeyAxiom;
import org.semanticweb.owlapi.model.OWLNegativeDataPropertyAssertionAxiom;
import org.semanticweb.owlapi.model.OWLNegativeObjectPropertyAssertionAxiom;
import org.semanticweb.owlapi.model.OWLObjectOneOf;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.model.OWLSubDataPropertyOfAxiom;
import org.semanticweb.owlapi.model.OWLSubPropertyChainOfAxiom;
import org.semanticweb.owlapi.profiles.OWL2ELProfile;
import org.semanticweb.owlapi.profiles.OWLProfile;
import org.semanticweb.owlapi.profiles.OWLProfileReport;
import org.semanticweb.owlapi.profiles.OWLProfileViolation;
import org.semanticweb.owlapi.profiles.UseOfDataOneOfWithMultipleLiterals;
import org.semanticweb.owlapi.profiles.UseOfIllegalAxiom;
import org.semanticweb.owlapi.profiles.UseOfIllegalClassExpression;
import org.semanticweb.owlapi.profiles.UseOfObjectOneOfWithMultipleIndividuals;
import org.semanticweb.owlapi.util.OWLObjectPropertyManager;
import org.semanticweb.owlapi.util.OWLOntologyWalker;
import org.semanticweb.owlapi.util.OWLOntologyWalkerVisitor;

public class SHOELProfile implements OWLProfile {

	@Override
	public OWLProfileReport checkOntology(OWLOntology ontology) {
		OWL2ELProfile profile = new OWL2ELProfile();
		OWLProfileReport report = profile.checkOntology(ontology);
		Set<OWLProfileViolation> violations = new HashSet<OWLProfileViolation>();
        violations.addAll(report.getViolations());
        OWLOntologyWalker ontologyWalker = new OWLOntologyWalker(ontology.getImportsClosure());
        SHOELProfileObjectVisitor visitor = new SHOELProfileObjectVisitor(ontologyWalker, ontology.getOWLOntologyManager());
        ontologyWalker.walkStructure(visitor);
        violations.addAll(visitor.getProfileViolations());
		return new OWLProfileReport(this, violations);
	}

	@Override
	public String getName() {
		// TODO Auto-generated method stub
		return "SHOEL";
	}
	
	 protected class SHOELProfileObjectVisitor extends OWLOntologyWalkerVisitor<Object> {

	        private final OWLOntologyManager man;

	        private OWLObjectPropertyManager propertyManager;

	        private final Set<OWLProfileViolation> profileViolations = new HashSet<OWLProfileViolation>();

	        public SHOELProfileObjectVisitor(OWLOntologyWalker walker, OWLOntologyManager man) {
	            super(walker);
	            this.man = man;
	        }

	        public Set<OWLProfileViolation> getProfileViolations() {
	            return new HashSet<OWLProfileViolation>(profileViolations);
	        }

	        @SuppressWarnings("unused")
			private OWLObjectPropertyManager getPropertyManager() {
	            if (propertyManager == null) {
	                propertyManager = new OWLObjectPropertyManager(man, getCurrentOntology());
	            }
	            return propertyManager;
	        }


	        
	        @Override
			public Object visit(OWLDataProperty p) {
	            profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }

	       
	        @Override
			public Object visit(OWLObjectOneOf desc) {
	            if (desc.getIndividuals().size() != 1) {
	                profileViolations.add(new UseOfObjectOneOfWithMultipleIndividuals(getCurrentOntology(), getCurrentAxiom(), desc));
	            }
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLDataHasValue desc) {
	                profileViolations.add(new UseOfIllegalClassExpression(getCurrentOntology(), getCurrentAxiom(), desc));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLDataSomeValuesFrom desc) {
	                profileViolations.add(new UseOfIllegalClassExpression(getCurrentOntology(), getCurrentAxiom(), desc));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLDataIntersectionOf desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLSubDataPropertyOfAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLEquivalentDataPropertiesAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        

	        
	        @Override
			public Object visit(OWLDataPropertyDomainAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        

	        
	        
	        
	        @Override
			public Object visit(OWLDataPropertyAssertionAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLNegativeDataPropertyAssertionAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLNegativeObjectPropertyAssertionAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLFunctionalDataPropertyAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	        
	        @Override
			public Object visit(OWLHasKeyAxiom desc) {
	                profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }
	       
	        


	        
	        @Override
			public Object visit(OWLDataOneOf node) {
	               profileViolations.add(new UseOfDataOneOfWithMultipleLiterals(getCurrentOntology(), getCurrentAxiom(), node));
	            return null;
	        }

	        

	        @Override
			public Object visit(OWLSubPropertyChainOfAxiom axiom) {
	        	profileViolations.add(new UseOfIllegalAxiom(getCurrentOntology(), getCurrentAxiom()));
	            return null;
	        }

	        @Override
			public Object visit(OWLOntology ontology) {
	            propertyManager = null;
	            return null;
	        }
	    }

	@Override
	public IRI getIRI() {
		// TODO Auto-generated method stub
		return null;
	}


}