package uk.ac.ox.eolo.reasoner;

import java.util.HashSet;
import java.util.Set;

import uk.ac.ox.cs.JRDFox.JRDFStoreException;
import uk.ac.ox.cs.JRDFox.store.DataStore;
import uk.ac.ox.cs.JRDFox.store.TupleIterator;
import uk.ac.ox.eolo.URIUtils;
import uk.ac.ox.eolo.model.EoloPrefixes;

public class DatalogReachability {

	private Set<String> marked; // marked[v] = true if v is reachable
	// from source (or sources)
	private int count; // number of vertices reachable from s

	/**
	 * Computes the vertices in digraph <tt>G</tt> that are reachable from the
	 * source vertex <tt>s</tt>.
	 * 
	 * @param G
	 *            the digraph
	 * @param s
	 *            the source vertex
	 * @throws JRDFStoreException 
	 */
	public DatalogReachability(DataStore store, String s, Set<String> L) throws JRDFStoreException {
		marked = new HashSet<String>();
		dfs(store, s, L);
	}
	
	/**
	 * Computes the vertices in digraph <tt>G</tt> that are reachable from the
	 * source vertex <tt>s</tt>.
	 * 
	 * @param G
	 *            the digraph
	 * @param s
	 *            the source vertex
	 * @throws JRDFStoreException 
	 */
	public DatalogReachability(DataStore store, String s) throws JRDFStoreException {
		marked = new HashSet<String>();
		dfs(store, s);
	}

	public Set<String> adj(DataStore store, String v, Set<String> L) throws JRDFStoreException {
		Set<String> adj = new HashSet<String>();
		String query = getLReachabilityQuery(v, L);
		TupleIterator tupleIterator = null;
		try {
			tupleIterator = store.compileQuery(query);
			for (tupleIterator.open(); tupleIterator.isValid(); tupleIterator.getNext()) {
				adj.add(tupleIterator.getGroundTerm(0).toString());
			}
			return adj;
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
	}
	
	public Set<String> adj(DataStore store, String v) throws JRDFStoreException {
		Set<String> adj = new HashSet<String>();
		String query = getLReachabilityQuery(v);
		TupleIterator tupleIterator = null;
		try {
			tupleIterator = store.compileQuery(query);
			for (tupleIterator.open(); tupleIterator.isValid(); tupleIterator.getNext()) {
				adj.add(tupleIterator.getGroundTerm(0).toString());
			}
			return adj;
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
	}

	private void dfs(DataStore store, String v, Set<String> L) throws JRDFStoreException {
		if (count != 0)
			marked.add(v);
		count++;
		for (String w : adj(store, v, L)) {
			if (!marked.contains(w))
				dfs(store,w, L);
		}
	}
	
	private void dfs(DataStore store, String v) throws JRDFStoreException {
		count++;
		marked.add(v);
		for (String w : adj(store, v)) {
			if (!marked.contains(w))
				dfs(store,w);
		}
	}

	/**
	 * Is there a directed path from the source vertex (or any of the source
	 * vertices) and vertex <tt>v</tt>?
	 * 
	 * @param v
	 *            the vertex
	 * @return <tt>true</tt> if there is a directed path, <tt>false</tt>
	 *         otherwise
	 */
	public boolean marked(String v) {
		return marked.contains(v);
	}

	/**
	 * Returns the number of vertices reachable from the source vertex (or
	 * source vertices).
	 * 
	 * @return the number of vertices reachable from the source vertex (or
	 *         source vertices)
	 */
	public int count() {
		return count;
	}
	
	private String getLReachabilityQuery(String start, Set<String> L) {
		String ruleBody = null;
		for (String r : L) {
			ruleBody = start + " " + "<" + EoloPrefixes.DIRECT_URI + URIUtils.shorten(r) +">" + " ?y  .";
		}		
		String res = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n select ?y WHERE { " + ruleBody + " ?y rdf:type <" + EoloPrefixes.AUX_URI + "Aux> .}";
		return res;
	}
	
	private String getLReachabilityQuery(String start) {
		String res= "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n select ?y WHERE { " + start + " <" + EoloPrefixes.DIRECT_URI  +"any>" + "" +
				" ?y  ." + " ?y rdf:type <" + EoloPrefixes.AUX_URI + "Aux> .}";
		return res;
	}
	
}
