package uk.ac.ox.eolo.reasoner;

import org.semanticweb.owlapi.model.AxiomType;
import org.semanticweb.owlapi.model.OWLOntology;

import uk.ac.ox.cs.JRDFox.JRDFStoreException;
import uk.ac.ox.cs.JRDFox.Prefixes;
import uk.ac.ox.cs.JRDFox.store.DataStore;
import uk.ac.ox.cs.JRDFox.store.Parameters;
import uk.ac.ox.cs.JRDFox.store.TupleIterator;
import uk.ac.ox.eolo.model.EoloPrefixes;

public class StoreStatistics {
	
	static final String SW_PREFIX = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n"
			+ "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n";
	
	public static String ontoStats(OWLOntology rootOntology) {
		String res = "";
		res += "Number of class assertions: "
				+ rootOntology.getAxiomCount(AxiomType.CLASS_ASSERTION) + "\n";
		res += "Number of object proprieries: "
				+ rootOntology
						.getAxiomCount(AxiomType.OBJECT_PROPERTY_ASSERTION) +"\n";
		res += "Number of individuals: "
				+ rootOntology.getIndividualsInSignature().size();
		return res;
	}
	
	public static String computeStatistics(DataStore store) throws JRDFStoreException {
		return "Total number of assertions: " +store.getTriplesCount() +
				"\n\n Total number of inds: " + getTotalCount(store) +
				"\n Total number of auxiliary individuals: " + getAuxiliaryCount(store)+ 
				"\nTotal number of unary " + getTotalUnaryCount(store) +
				"\n Total number of unary facts on auxiliary individuals: " + getUnaryCount(store)+ 
				"\nTotal number of binary " + getTotalBinaryCount(store) +
				"\n Total number of binary facts on auxiliary individuals: " + getBinaryCount(store);
	}
	
	
	public static long getTotalCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT  ?X WHERE { ?X rdf:type owl:Thing .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = true;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	public static long getAuxiliaryCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT ?X WHERE { ?X rdf:type " + EoloPrefixes.AUX_PREFIX +"Aux .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = true;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	public static long getTotalUnaryCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT ?X ?Y WHERE { ?X rdf:type ?Y. ?X rdf:type owl:Thing .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = true;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	public static long getTotalBinaryCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT ?X ?P ?Y  WHERE { ?X ?P ?Y. ?X rdf:type owl:Thing. ?Y rdf:type owl:Thing .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = true;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	public static long getUnaryCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT  ?X ?Y WHERE { ?X rdf:type ?Y. ?X rdf:type " + EoloPrefixes.AUX_PREFIX +"Aux .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = false;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	
	public static long getBinaryCount(DataStore store) throws JRDFStoreException {
		TupleIterator tupleIterator=null;
		long res =0;
		String query = SW_PREFIX + EoloPrefixes.getDeclaration()+
				"SELECT ?X ?P ?Y WHERE { ?X ?P ?Y. ?X rdf:type owl:Thing. ?Y rdf:type " + EoloPrefixes.AUX_PREFIX +"Aux .}";
		try {
			Parameters params = new Parameters();
			params.m_expandEquality = true;
			tupleIterator = store.compileQuery(query, new Prefixes(), params);
			for ( long multiplicity = tupleIterator.open(); multiplicity != 0; multiplicity = tupleIterator
					.getNext()) {
				res++;
			}
		} finally {
			if (tupleIterator != null)
				tupleIterator.dispose();
		}
		return res;
	}
	
	



}
