package uk.ac.ox.eolo.reasoner;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;

import uk.ac.ox.eolo.cqparser.ConjunctiveQueryParser;
import uk.ac.ox.eolo.model.ConjunctiveQuery;

public class Tester {
	public static int[] ontos = { 5,10,20};
	public static int[] queries = {2};
	public static String ontoFolder = "testfiles/LUBM-SHOEL/ontologies/";
	public static String queryFolder = "testfiles/LUBM-SHOEL/query/";
	public static String logfile = "testresults/LUBM-SHOEL/";

	public static boolean evaluateQueries = true;
	public static boolean stats = false;

	public static void main(String[] args) {

		for (int i : ontos) {
			EOLO reasoner = null;
			PrintWriter writer = null;
			ResultCollector res = null;
			try {
				File logFile = new File(ontoFolder + i + "/log.txt");
				if (!logFile.exists())
					logFile.createNewFile();
				writer = new PrintWriter(new BufferedWriter(new FileWriter(logFile)), true);
				System.out.println("#############################################");
				System.out.println("START ONTO " + i);
				writer.println("#############################################");
				writer.println("START ONTO " + i);
				String ontoname = ontoFolder + i + "/LUBM-SHOEL-" + i
						+ "univ.owl";
				OWLOntologyManager man = OWLManager.createOWLOntologyManager();
				System.out.println("loading ontology file " + ontoname);
				writer.println("loading ontology file " + ontoname);
				OWLOntology root = man
						.loadOntologyFromOntologyDocument(new File(ontoname));
				File dataFile = new File(ontoFolder + i + "/data.ttl");
				File ruleFile = new File(ontoFolder + i + "/rules.dlog");
				if (!dataFile.exists())
					dataFile.createNewFile();
				if (!ruleFile.exists())
					ruleFile.createNewFile();
				System.out.println("initializing RDFox...");
				writer.println("initializing RDFox...");
				double start = System.currentTimeMillis();
				reasoner = EOLO.createReasoner(root, dataFile, ruleFile, true);
				double end = System.currentTimeMillis();
				if(stats) {
					System.out.println("...done. It took me " + ((end - start) / 1000)
						+ " seconds.");
					System.out.println(StoreStatistics.ontoStats(root));
					System.out.println(StoreStatistics.computeStatistics(reasoner
						.getStore()));
				}
				root =null;
				man = null;
				if (evaluateQueries) {
					for (int j : queries) {
						System.out.println("---------------------");
						System.out.println("START QUERY ");
						writer.println("---------------------");
						writer.println("START QUERY ");
						String queryname = queryFolder + "query" + j + ".txt";
						ConjunctiveQuery q = new ConjunctiveQueryParser(
								new File(queryname)).parse();
						System.out.println("evaluating query " + j);
						writer.println("evaluating query " + j);
						start = System.currentTimeMillis();
						res = reasoner.answerCQ(q);
						end = System.currentTimeMillis();
						System.out.println("..done in "
								+ Double.toString(((end - start) / 1000))
								+ " seconds.");
						System.out.println(res);
						writer.println("..done in "
								+ Double.toString(((end - start) / 1000))
								+ " seconds.");
						writer.println(res);
					}
				}
				System.out.println();
				writer.println("END ONTO ");
			} catch (IOException e) {
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} finally {
				if (writer != null)
					writer.close();
				if (reasoner != null)
					reasoner.dispose();
			}
		}

	}

}
